# frozen_string_literal: true

module Docker
  class ConfigGenerator < Rails::Generators::Base
    source_root File.expand_path('../templates', __FILE__)

    def setup
      config = file?('docker/dockerconfig.json') ? JSON.parse(File.read('docker/dockerconfig.json')) : {}
      @base_image_name = ask_with_default('Which Image do you want to from(Base Image)?', config['base_name'] || default_base_image_name)
      @author = ask_with_default('Who is the Author', config['author'] || ENV['USER'])
      @service_name = ask_with_default('What is the Service Name', config['service_name'] || 'app')
      db_type = %w[mongo mysql postgres]
      say('what type of DB you are using:')
      print_table(db_type.map.with_index { |a, i| [i + 1, a] })
      default_choice = if file?('config/mongoid.yml')
                         1
                       else
                         2
      end
      db_select = ask_with_default('choice:', default_choice).to_i
      @db_type = db_type[db_select - 1]
      @db = db_content(@db_type)
      @service_image_name = ask_with_default('Service Image Name ', config['image_name'] || default_service_image_name)
      @docker_repo_url = ask_with_default('Your docker repo', config['docker_repo'] || default_docker_repo)
      @secret = `rake secret`.chop
    end

    def create_docker_config
      say('creating docker config')
      template 'dockerconfig.json', 'docker/dockerconfig.json'
    end

    def create_docker_file
      say('creating docker file')
      @db_config_file = if @db_type == 'mongo'
                          'mongoid'
                        else
                          'database'
        end
      template 'Dockerfile', 'docker/Dockerfile'
    end

    def create_docker_ignore
      content = <<~CODE
        config/database.yml
        config/mongoid.yml
        Dockerfile
        docker-compose*.yml
        docker/
        lib/tasks/docker.rake
CODE
      filename = '.dockerignore'
      if file? filename
        append_to_file filename, content
      else
        create_file filename, content
      end
    end

    def create_git_ignore
      content = <<~CODE
        # See https://help.github.com/articles/ignoring-files for more about ignoring files.
        #
        # If you find yourself ignoring temporary files generated by your text editor
        # or operating system, you probably want to add a global ignore instead:
        #   git config --global core.excludesfile '~/.gitignore_global'

        # Ignore bundler config.
        /.bundle

        # Ignore all logfiles and tempfiles.
        .rake_tasks
        /log/*
        !/log/.keep
        /tmp
        *~
        .#*
        public/assets
        config/settings.local.yml
        config/settings/*.local.yml
        config/environments/*.local.yml
        config/mongoid.yml
        cache
        spring/*
        vendor/*
CODE
      filename = '.gitignore'
      if file? filename
        append_to_file filename, content
      else
        create_file filename, content
      end
      end

    def create_docker_composer_file
      say('creating docker composer file')
      template 'docker-compose.yml', 'docker/docker-compose.yml'
    end

    def create_db_config_for_docker
      if @db_type == 'mongo'
        template 'mongoid.yml', 'config/mongoid.docker.yml'
      else
        template "database.#{@db_type}.yml", 'config/database.docker.yml'
      end
    end

    def create_docker_composer_debug_file
      say('creating docker composer debug file')
      template 'docker-compose.debug.yml', 'docker/docker-compose.debug.yml'
    end

    def create_docker_task
      say('creating docker task')
      template 'docker.rake', 'lib/tasks/docker.rake'
    end

    private

    def file?(path)
      File.exist?(path)
    end

    def ask_with_default(statement, default_value)
      answer = ask("#{statement}[#{default_value}]:", Thor::Shell::Color::BLUE)
      answer.present? ? answer : default_value
    end

    def default_base_image_name
      'shaoyang/ruby:alpine'
    end

    def default_service_image_name
      File.basename(Dir.getwd)
    end

    def default_docker_repo
      ENV['DEFAULT_DOCKER_REPO'] || "docker.io/#{@author}"
    end

    def db_content(db_type)
      temp_dir = File.expand_path('../templates', __FILE__)
      File.read("#{temp_dir}/#{db_type}.erb").chop
    end
   end
end
